# Proficio Toolbox

This is a wrapper for the Proficio that lies on top of libbarrett.
It includes libbarrett based systems and tools that are specific to the Proficio. The demos are not guaranteed to run on any other robot other than Proficio.

## Pre-Requisite
* Install libbarrett from [here] (https://github.com/BarrettTechnology/libbarrett.git). (Only runs with versions >= 1.2.4)
* sudo apt-get install libcgal-dev

## Build, compile and install
* cd ~/
* git clone https://github.com/BarrettTechnology/proficio_toolbox.git
* cd proficio_toolbox
* mkdir build
* cd build
* cmake ../
* make
* sudo make install

## Programming

The programming is similar to that of libbarrett except for few minor changes.
* use 
```c++
  #include <proficio/systems/utilities.h>
```
* use 
```c++
  #include <proficio/standard_proficio_main.h> 
```
   instead of 
```c++
  #include <barrett/standard_main_function.h>
```
* use 
```c++
  template <size_t DOF>
  int proficio_main(int argc, char** argv, barrett::ProductManager& pm,
               barrett::systems::Wam<DOF>& wam, config side) {
```
   instead of 
```c++
  template <size_t DOF>
  int wam_main(int argc, char** argv, ProductManager& pm, systems::Wam<DOF>& wam) {
```
* Include
```c++
#define NO_CONTROL_PENDANT
```
   only if you would like to skip the use of control pendant buttons (Shift activate and Shift idle). 

In such a case, you may have to zero the robot each time you power it (each time the Pucks are turned on) by doing the following steps

  * Move the robot to its home position and open joint 3 until the arm is fully extended.

  * Open a terminal and run the following, with side being left or right based on the robot's configuration
  $ homecheck <side>

<b> Please do not use this unless you know what you are doing. This can lead to undesirable safety issues. It is highly recommended to remove the above macro from all the demos that are shipped in this repository</b> 

Please contact support@barrett.com for additional information.

For examples on programming and creating CMakeLists, please refer to the examples under proficio_demos.
