/*
	Copyright 2015 Barrett Technology <support@barrett.com>

	This file is part of libbarrett.

	This version of libbarrett is free software: you can redistribute it
	and/or modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation, either version 3 of the
	License, or (at your option) any later version.

	This version of libbarrett is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this version of libbarrett.  If not, see
	<http://www.gnu.org/licenses/>.

*/

/** Joint Torques Saturation system that limits the torques sent to the robot.
 *
 * @file jtSat.h
 * @date Dec 09, 2015
 * @author Hariharasudan Malaichamee
 */

#include <barrett/systems.h>
#include <barrett/systems/abstract/system.h>
#include <barrett/units.h>

template<size_t DOF>
class jtSat : public barrett::systems::SingleIO<typename barrett::units::JointTorques<DOF>::type, typename barrett::units::JointTorques<DOF>::type> {
	BARRETT_UNITS_TEMPLATE_TYPEDEFS(DOF);

public:
	jtSat(const jt_type limits, const std::string& sysName = "jtSat") :
		barrett::systems::SingleIO<jt_type, jt_type>(sysName), torqueLimits(limits){}

	virtual ~jtSat() { this->mandatoryCleanUp(); }

	void setJtLimit(jt_type &jtLimits) {
		torqueLimits = jtLimits;
	}

protected:

	virtual void operate() {

		jointTorques = this->input.getValue();		
		minRatio = (torqueLimits.cwise() / (jointTorques.cwise().abs())).minCoeff(&index);
		if (minRatio < 1.0) {
    			torques = minRatio * jointTorques;
  		} else {
    			torques = jointTorques;
  		}
		this->outputValue->setData(&torques);
	}

	jt_type torqueLimits, jointTorques, torques;
	int index;
  	double minRatio;

private:
	DISALLOW_COPY_AND_ASSIGN(jtSat);
};


