/*
 * forward_kin.h
 *
 *  Created on: Sep 16, 2015
 *      Author: hm
 */
#include <barrett/cdlbt/gsl.h>
#include <barrett/cdlbt/kinematics.h>
#include <barrett/cdlbt/calgrav.h>

#include <barrett/config.h>

#include <barrett/units.h>

#include <math.h>
#include <vector>

#ifndef FORWARD_KIN_H_
#define FORWARD_KIN_H_

template <size_t DOF>
class ForwardKin{
	BARRETT_UNITS_TEMPLATE_TYPEDEFS(DOF);
public:
	ForwardKin(struct bt_kinematics * kn, config_setting_t * wld): kin(kn), world(wld){

		// Populate world to base transform
		config_setting_t * world_row;
		for (size_t i = 0; i < 4; ++i) {
			world_row = config_setting_get_elem( world, i );
			for(size_t j = 0; j < 4; ++j){
				double val;
				bt_gsl_config_get_double(config_setting_get_elem( world_row, j ), &val);
				Base_to_World(i,j) = val;
			}
		}
	}
	cp_type findFwKinPosition(jp_type& ang){
		Eigen::Matrix4f Rz, Tz, Tx, Rx, Transf;
		std::vector<double> pos;
		pos.resize(DOF);

		cart_pos.setZero();

		std::vector<Eigen::Matrix4f> A;
		A.resize(DOF);

		Transf_Mat = Eigen::MatrixXf::Identity(4,4);
		// Construct Transformation matrices for each frame that involves Rz, Tz, Tx, Rx
		for(size_t i = 0; i < DOF; ++i)
		{
			A[i] << cos(ang[i]), -sin(ang[i])*cos(kin->link[i]->alpha), sin(ang[i])*sin(kin->link[i]->alpha), kin->link[i]->a * cos(ang[i]),
					sin(ang[i]), cos(ang[i])*cos(kin->link[i]->alpha), -cos(ang[i])*sin(kin->link[i]->alpha), kin->link[i]->a * sin(ang[i]),
					0, sin(kin->link[i]->alpha), cos(kin->link[i]->alpha), kin->link[i]->d,
					0, 0, 0, 1;
		}

		// Homogeneous transformation from Fram 0 to Frame n
		for(size_t i = 0; i < DOF; ++i)
		{
			Transf_Mat = Transf_Mat * A[i];
		}

		// Convert the coordinates in the base frame
		Transf_Mat = Base_to_World * Transf_Mat;

		cart_pos << Transf_Mat(0,3), Transf_Mat(1,3), Transf_Mat(2,3);
		return cart_pos;
	}
protected:
		struct bt_kinematics * kin;
		config_setting_t * world;
		Eigen::Matrix4f Transf_Mat, Base_to_World;
		cp_type cart_pos;
};


#endif /* FORWARD_KIN_H_ */
