#!/usr/bin/python
import sys
import time
import math
import select
import struct
import numpy
import Image
from socket import *
from visual import *

# args
if len(sys.argv) == 2:
    remoteHost = sys.argv[1]
else:
    print "Usage: %s <WAM IP address>" % sys.argv[0]
    exit()

# Networking
PORT_SRC = 5556
PORT_DEST = 5557
MSG_FORMAT = "dddddddddd"  # messages contain 10 doubles - 3 Proxy Position - 3 Proxy Velocity - 3 Ball Position - 1 Button Press
MSG_FORMAT_SEND = "ddddddddddd"
MSG_SIZE = struct.calcsize(MSG_FORMAT)

sock = socket(AF_INET, SOCK_DGRAM)
# sock.setblocking(0)
sock.bind(('', PORT_SRC))
sock.connect((remoteHost, PORT_DEST))

# Visuals
xMin, xMax = 0.3, 0.7
yMin, yMax = -0.25, 0.25
zMin, zMax = 0.0, 0.5

xSize = xMax - xMin
ySize = yMax - yMin
zSize = zMax - zMin

xMiddle = xMin + (xSize / 2)
yMiddle = yMin + (ySize / 2)
zMiddle = zMin + (zSize / 2)

proxyPos = (xMiddle, yMiddle, zMiddle)
proxySize = 0.03
proxyColor = (0.0, 1.0, 0.0)
proxyVel = (0.0, 0.0, 0.0)

ballPos = (0.35, 0.2, 0.75)
ballSize = 0.075

buttonPress = False

# Setup the scene
scene.fullscreen = True
scene.center = (0.0, yMiddle, zMiddle + 0.15)  # 0.4
scene.fov = math.pi / 2.0;
scene.scale = (1.3, 1.3, 1.3)
scene.forward = (-0.00250513, 0.517794, -0.855502)

f = frame()
f.rotate(angle= -pi / 2.0, axis=(1, 0, 0), origin=(0, 0, 0))
f.rotate(angle= -pi / 2.0, axis=(0, 1, 0), origin=(0, 0, 0))
f.rotate(angle=pi * 0.2, axis=(1, 0, 0), origin=(0, 0, 0))

wall_size = 0.002;

cord_name = "textures/corduroy"
imC = Image.open(cord_name + ".png")
materials.saveTGA(cord_name, imC)

barrett_name = "textures/barrett_logo"
imB = Image.open(barrett_name + ".png")
imB.resize((1024, 256), Image.ANTIALIAS)
materials.saveTGA(barrett_name, imB)

magnet_name = "textures/magnet"
imM = Image.open(magnet_name + ".png")
materials.saveTGA(magnet_name, imM)

corduroy = materials.texture(data=materials.loadTGA(cord_name), mapping="rectangular")
barrett = materials.texture(data=materials.loadTGA(barrett_name), mapping="rectangular")
magnet = materials.texture(data=materials.loadTGA(magnet_name), mapping="rectangular")

floor = box(pos=(xMiddle, yMiddle, zMin), length=xSize, height=ySize, width=wall_size, frame=f, material=corduroy)
barrett_wall = box(pos=(xMin + 0.01, yMiddle, zMiddle), length=wall_size, height=ySize - 0.1, width=zSize / 4.8, frame=f, material=barrett)
barrett_wall.rotate = (pi / 4, (1, 0, 0), barrett_wall.pos)
back = box(pos=(xMin, yMiddle, zMiddle), length=wall_size, height=ySize, width=zSize, frame=f, color=(47.0 / 255.0, 40.0 / 255.0, 228.0 / 255.0))
left = box(pos=(xMiddle, yMin, zMiddle), length=xSize, height=wall_size, width=zSize, frame=f, color=color.yellow)
right = box(pos=(xMiddle, yMax, zMiddle), length=xSize, height=wall_size, width=zSize, frame=f, color=(1.0, 0.7, 0.2))
magnet_wall = cylinder(pos=(xMiddle, yMax, zMiddle), axis=(0,-0.0012,0), radius=0.15, frame=f, material=magnet)
ceiling = box(pos=(xMiddle, yMiddle, zMax), length=xSize, height=ySize, width=wall_size, frame=f, color=(228.0 / 255.0, 40.0 / 255.0, 184.0 / 255.0))

left_button = cylinder(pos=(xMiddle, yMin, zMiddle), axis=(0.0, 0.01, 0.0), radius=0.15, frame=f, color=color.red)

proxy = sphere(pos=(proxyPos[0:3]), radius=proxySize, color=color.green, frame=f)
ball = sphere(pos=(ballPos[0:3]), radius=ballSize, material = materials.earth, frame=f)

while True:
   msg = struct.unpack(MSG_FORMAT, sock.recv(MSG_SIZE))
   proxy.pos = (msg[0],msg[1],msg[2])
   proxyVel = (msg[3:6])
   ball.pos = (msg[6],msg[7],msg[8])
   buttonPress = (bool)(msg[9])
   if buttonPress == True:
       left_button.axis = (0.0, 0.004, 0.0)
       left.color = (color.green)
   else:
       left_button.axis = (0.0, 0.01, 0.0)
       left.color = (color.yellow)
   if  abs((proxy.pos[0]-proxySize) - back.pos[0]) < 0.01:
       back.color = (117.0 / 255.0, 69.0 / 255.0, 27.0 / 255.0)
   else:
       back.color = (47.0 / 255.0, 40.0 / 255.0, 228.0 / 255.0)
   if  abs((proxy.pos[2]+proxySize) - ceiling.pos[2]) < 0.01:
       ceiling.color = (5.0 / 255.0, 207.0 / 255.0, 255.0 / 255.0)
   else:
       ceiling.color = (228.0 / 255.0, 40.0 / 255.0, 184.0 / 255.0)
   if scene.kb.keys:  # If any input was pressed
       s = scene.kb.getkey()
       if s == 'up':
            gcomp = 1
       elif s == 'down':
	    gcomp = 2
       elif s == 'r':
	    gcomp = 0
       sock.send(struct.pack(MSG_FORMAT_SEND,gcomp,1,1,1,1,1,1,1,1,1,1))
       sock.send(struct.pack(MSG_FORMAT_SEND,100,1,1,1,1,1,1,1,1,1,1))       
sock.close()

